﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading.Tasks;

namespace entropy
{
    class Program
    {
        private static StreamReader inputData = new StreamReader("input.txt");
        private static StreamReader inputData2 = new StreamReader("input3.txt");
        private static StreamWriter outputData = new StreamWriter("output.txt");
        private static StreamWriter outputData3 = new StreamWriter("output3.txt");
        private static StreamWriter outputData4 = new StreamWriter("output4.txt");

        private static List<string> listOfNameVectors = new List<string>();
        private static List<List<double>> listOfVectors = new List<List<double>>();
        private static List<List<double>> listOfVectorsCopy = new List<List<double>>();

        private static List<string> markedVectors = new List<string>();
        private static List<List<double>> markedCenters = new List<List<double>>();
        private static List<List<string>> markedGroups = new List<List<string>>();
        private static List<string> markedEuqvalVectors = new List<string>();

        private static List<List<string>> InitialGroups = new List<List<string>>();
        private static List<List<string>> Groups = new List<List<string>>();
        private static List<List<string>> Clusters = new List<List<string>>();
        private static List<List<double>> centersOfClusters = new List<List<double>>();

        private static List<List<double>> entropyForEachGroup = new List<List<double>>();
        private static List<double> bestFiveEntropyForEachGroup = new List<double>();
        private static List<List<double>> distanceForEachQuery = new List<List<double>>();
        private static List<double> bestFiveDistanceForEachQuery = new List<double>();
      
        private static bool IsEuqvalQuery(List<double> q1, List<double> q2)
        {
            for (int i = 0; i < 7; i++)
            {
                if (q1[i] != q2[i])
                {
                    return false;
                }
            }
            return true;
        }
        private static void SearchInitialGr()
        {
            foreach (string qQur in listOfNameVectors)
            {
                if (!markedEuqvalVectors.Contains(qQur))
                {
                    List<string> qurGr = new List<string>();
                    qurGr.Add(qQur);
                    markedEuqvalVectors.Add(qQur);
                    foreach (string qSear in listOfNameVectors)
                    {
                        if (!markedEuqvalVectors.Contains(qSear)
                         && IsEuqvalQuery(listOfVectors[listOfNameVectors.IndexOf(qQur)], listOfVectors[listOfNameVectors.IndexOf(qSear)]))
                        {

                            qurGr.Add(qSear);
                            markedEuqvalVectors.Add(qSear);

                        }
                    }
                    InitialGroups.Add(qurGr);
                }

            }
        }
        private static void PrintCenters()
        {
            foreach (List<double> center in centersOfClusters)
            {
                foreach (double value in center)
                {
                    outputData.Write("{0} ", value);
                }
                outputData.WriteLine();
            }
            outputData.Close();
        }
        private static void CreateCenters()
        {

            foreach (List<string> cluster in Clusters)
            {
                double[] curResult = new double[6];//2387
                foreach (string query in cluster)
                {
                    for (int i = 0; i < 6; i++)//2387
                    {
                        curResult[i] += listOfVectors[listOfNameVectors.IndexOf(query)][i];
                    }
                }
                for (int i = 0; i < 6; i++)//2387
                {
                    curResult[i] = curResult[i] / cluster.Count();
                }
                centersOfClusters.Add(curResult.ToList());
            }
        }
        private static double SeparateProbability(List<double> X, List<string> xGroup)
        {
            List<List<double>> xGr = new List<List<double>>();
            foreach (string query in xGroup)
            {
                xGr.Add(listOfVectors[listOfNameVectors.IndexOf(query)]);
            }

            int frequencyOfX = xGr.Count(p => p.Equals(X));
            int numberOfElements = xGroup.Count;

            return (float)frequencyOfX / numberOfElements;
        }

        private static double JointProbability(List<double> X, List<double> Y, List<string> xGroup, List<string> yGroup)
        {
            int frequencyOfX = 0;
            int frequencyOfY = 0;
            foreach (string el in xGroup)
            {
                int count = 0;
                for (int i = 0; i < 7; i++)
                {
                    if (listOfVectors[listOfNameVectors.IndexOf(el)][i] == X[i])
                    {
                        count++;
                    }
                }
                if (count == 7)
                {
                    frequencyOfX++;
                }
            }

            foreach (string el in yGroup)
            {
                int count = 0;
                for (int i = 0; i < 7; i++)
                {
                    if (listOfVectors[listOfNameVectors.IndexOf(el)][i] == Y[i])
                    {
                        count++;
                    }
                }
                if (count == 7)
                {
                    frequencyOfY++;
                }
            }

            int numberOfCombination = (xGroup.Count() + yGroup.Count()) * (xGroup.Count() + yGroup.Count() - 1) / 2;

            return (float)frequencyOfX * frequencyOfY / numberOfCombination;
        }

        private static long Factorial(int n)
        {
            return (n == 0) ? 1 : n * Factorial(n - 1);
        }

        private static double Entropy(List<string> xGroup)
        {
            double result = 0;

            List<List<double>> valuesOfXGroup = new List<List<double>>();

            foreach (string query in xGroup)
            {
                if (!valuesOfXGroup.Contains(listOfVectors[listOfNameVectors.IndexOf(query)]))
                {
                    valuesOfXGroup.Add(listOfVectors[listOfNameVectors.IndexOf(query)]);
                }
            }

            foreach (List<double> xValue in valuesOfXGroup)
            {
                double separateProbability = SeparateProbability(xValue, xGroup);
                if (separateProbability != 0)
                {
                    result += separateProbability * Math.Log(separateProbability, 2);
                }
            }

            return -result;
        }
        private static double JointEntropy(List<string> xGroup, List<string> yGroup)
        {
            double result = 0;

            List<string> completeX = new List<string>();
            List<string> completeY = new List<string>();

            foreach (string xValue in xGroup)
            {
                if (!completeX.Contains(xValue))
                {
                    completeX.Add(xValue);
                    foreach (string yValue in yGroup)
                    {
                        if (!completeY.Contains(yValue))
                        {
                            completeY.Add(yValue);
                            double joinProbability = JointProbability(listOfVectors[listOfNameVectors.IndexOf(xValue)], listOfVectors[listOfNameVectors.IndexOf(yValue)], xGroup, yGroup);
                            if (joinProbability != 0)
                            {
                                result += joinProbability * Math.Log(joinProbability, 2);
                            }
                        }
                    }
                }
            }

            return -result;
        }

        private static double EntropyForGroup(List<string> xGroup, List<string> yGroup)
        {
            List<string> newGroup = new List<string>();
            newGroup.AddRange(xGroup);
            newGroup.AddRange(yGroup);
            List<string> marked = new List<string>();
            int numerOfComponents = listOfVectors[listOfNameVectors.IndexOf(xGroup[0])].Count();
            int numberOfElements = newGroup.Count();
            double[] entropyArray = new double[numerOfComponents];
            List<int> frequencyArray = new List<int>();
            for (int i = 0; i < numerOfComponents; i++)
            {
                marked.Clear();
                frequencyArray.Clear();
                for (int j = 0; j < numberOfElements; j++)
                {
                    if (!marked.Contains(newGroup[j]))
                    {
                        int curFriq = 1;
                        marked.Add(newGroup[j]);

                        for (int k = 0; k < numberOfElements; k++)
                        {
                            if (!marked.Contains(newGroup[k]))
                            {
                                if (listOfVectors[listOfNameVectors.IndexOf(newGroup[j])][i] == listOfVectors[listOfNameVectors.IndexOf(newGroup[k])][i])
                                {
                                    curFriq++;
                                    marked.Add(newGroup[k]);
                                }
                            }
                        }
                        frequencyArray.Add(curFriq);
                    }
                }
                foreach (int frequency in frequencyArray)
                {
                    entropyArray[i] += -(((float)frequency / numberOfElements) * Math.Log(((float)frequency / numberOfElements), 2));
                }
            }

            return entropyArray.Sum();
        }

        private static double EuclidDistance(List<string> X, List<string> Y)
        {
            List<double> centerOfX = new List<double>();
            List<double> centerOfY = new List<double>();


            double[] curResult = new double[7];
            foreach (string query in X)
            {
                for (int i = 0; i < 7; i++)
                {
                    curResult[i] += listOfVectors[listOfNameVectors.IndexOf(query)][i];
                }
            }
            for (int i = 0; i < 7; i++)
            {
                curResult[i] = curResult[i] / X.Count();
            }
            centerOfX = curResult.ToList();

            curResult = new double[7];
            foreach (string query in Y)
            {
                for (int i = 0; i < 7; i++)
                {
                    curResult[i] += listOfVectors[listOfNameVectors.IndexOf(query)][i];
                }
            }
            for (int i = 0; i < 7; i++)
            {
                curResult[i] = curResult[i] / Y.Count();
            }
            centerOfY = curResult.ToList();

            double result = 0;
            for (int i = 0; i < 7; i++)
            {
                result += Math.Pow((centerOfX[i] - centerOfY[i]), 2);
            }
            return Math.Sqrt(result);
        }

        private static void CreateVectors()
        {
            string currentLine = string.Empty;
            string curActivity = string.Empty;
            string curQueryName = string.Empty;
            int cursymbol = 0;
            while ((currentLine = inputData.ReadLine()) != null)
            {
                cursymbol = 0;
                curQueryName = string.Empty;
                while (currentLine[cursymbol] != ' ')
                {
                    curQueryName += currentLine[cursymbol];
                    cursymbol++;
                }
                cursymbol++;
                List<double> curList = new List<double>();
                while (cursymbol < currentLine.Length)
                {
                    curActivity = string.Empty;
                    while (cursymbol < currentLine.Length && currentLine[cursymbol] != ' ')
                    {
                        curActivity += currentLine[cursymbol];
                        cursymbol++;
                    }
                    cursymbol++;
                    curList.Add(Convert.ToDouble(curActivity));
                }
                listOfNameVectors.Add(curQueryName);
                listOfVectors.Add(curList);
            }
        }
        private static void Create()
        {
            string currentLine = string.Empty;
            string curActivity = string.Empty;
            string curQueryName = string.Empty;
            int cursymbol = 0;
            while ((currentLine != null))
            {
                List<List<double>> listOfVectors2 = new List<List<double>>();
                List<string> listOfNameVectors2 = new List<string>();
                while ((currentLine = inputData2.ReadLine()) != string.Empty && (currentLine != null))
                {

                    cursymbol = 0;
                    curQueryName = string.Empty;
                    while (currentLine[cursymbol] != ' ')
                    {
                        curQueryName += currentLine[cursymbol];
                        cursymbol++;
                    }
                    cursymbol++;
                    List<double> curList = new List<double>();
                    while (cursymbol < currentLine.Length)
                    {
                        curActivity = string.Empty;
                        while (cursymbol < currentLine.Length && currentLine[cursymbol] != ' ')
                        {
                            curActivity += currentLine[cursymbol];
                            cursymbol++;
                        }
                        cursymbol++;
                        curList.Add(Convert.ToDouble(curActivity));
                    }
                    listOfNameVectors2.Add(curQueryName);
                    listOfVectors2.Add(curList);
                }
                Groups.Add(listOfNameVectors2);
            }
        }

        static void CreateClusters()
        {
            for (int j = 0; j < listOfVectors.Count(); j++)
            {
                for (int i = 0; i < 7; i++)
                {
                    listOfVectors[j][i] = Math.Round(listOfVectors[j][i], 1);

                }
            }
            for (int i = 0; i < Groups.Count; i++)
            {
                List<double> curList = new List<double>();
                for (int j = 0; j < Groups.Count; j++)
                {
                    if (i != j)
                    {
                        curList.Add(JointEntropy(Groups[i], Groups[j]));
                    }
                    else
                    {
                        curList.Add(0);
                    }
                }
                entropyForEachGroup.Add(curList);
            }
            for (int j = 0; j < entropyForEachGroup.Count(); j++)
            {
                List<double> sorted = new List<double>();
                foreach (double el in entropyForEachGroup[j])
                {
                    sorted.Add(el);
                }
                sorted.Sort();
                List<double> curBest = new List<double>();
                int i = 0;
                int k = 0;
                while (i < 3 /*&& k < curBest.Count*/)
                {
                    if (!curBest.Contains(sorted[k]) && sorted[k] != 0)
                    {
                        curBest.Add(sorted[k]);
                        i++;
                    }
                    k++;
                }
                //bestFiveEntropyForEachGroup.Add(curBest);
            }
            List<string> currentCluster = new List<string>();
            for (int i = 0; i < Groups.Count; i++)
            {
                currentCluster = new List<string>();
                if (!markedGroups.Contains(Groups[i]))
                {
                    currentCluster.AddRange(Groups[i]);
                    markedGroups.Add(Groups[i]);
                    foreach (double best in bestFiveEntropyForEachGroup)
                    {
                        for (int j = 0; j < Groups.Count; j++)
                        {
                            if (entropyForEachGroup[i][j] == best && !markedGroups.Contains(Groups[j]))
                            {
                                currentCluster.AddRange(Groups[j]);
                                markedGroups.Add(Groups[j]);
                            }
                        }
                    }
                    Clusters.Add(currentCluster);
                }
            }
            int count = 0;
            foreach (List<string> cluster in Clusters)
            {
                outputData4.WriteLine(count);
                foreach (string el in cluster)
                {
                    foreach (double doub in listOfVectorsCopy[listOfNameVectors.IndexOf(el)])
                    {
                        outputData4.Write(doub + " ");
                    }
                    outputData4.WriteLine();
                }
                outputData4.WriteLine();
                count++;
            }
            outputData3.Close();
        }


        static void CreateClusters2()
        {
            listOfVectors = new List<List<double>>();
            for (int j = 0; j < listOfVectorsCopy.Count(); j++)
            {
                List<double> curList = new List<double>();
                for (int i = 0; i < 7; i++)
                {
                    curList.Add(Math.Round(listOfVectorsCopy[j][i], 2));

                }
                listOfVectors.Add(curList);
            }
            for (int i = 0; i < Groups.Count; i++)
            {
                List<double> curList = new List<double>();
                for (int j = 0; j < Groups.Count; j++)
                {
                    if (i < j)
                    {

                        curList.Add(EntropyForGroup(Groups[i], Groups[j]));
                    }

                    else
                    {
                        curList.Add(10000.0);
                    }
                }
                entropyForEachGroup.Add(curList);
            }
            int clustersCount = Groups.Count();
            List<int> markedGroup = new List<int>();

            while (clustersCount > 60)
            {
                //double min = 10000.0;
                List<List<string>> newGroups = new List<List<string>>();
                List<int> curMarkedGroup = new List<int>();

                bestFiveEntropyForEachGroup.Clear();
                List<double> sorted = new List<double>();
                for (int i = 0; i < entropyForEachGroup.Count(); i++)
                {

                    for (int j = 0; j < entropyForEachGroup.Count(); j++)
                    {
                        sorted.Add(entropyForEachGroup[i][j]);
                    }
                }
                sorted.Sort();
                List<double> curBest = new List<double>();
                int ih = 0;
                int kh = 0;
                while (ih < 1)
                {
                    if (sorted[kh] != 0 && !curBest.Contains(sorted[kh]))
                    {
                        curBest.Add(sorted[kh]);
                        ih++;
                    }
                    kh++;
                }
                bestFiveEntropyForEachGroup.AddRange(curBest);
                for (int i = 0; i < entropyForEachGroup.Count(); i++)
                {
                    for (int j = 0; j < entropyForEachGroup.Count(); j++)
                    {
                        if (i < j && bestFiveEntropyForEachGroup.Contains(entropyForEachGroup[i][j]))
                        {
                            if (curMarkedGroup.Contains(i) || curMarkedGroup.Contains(j) || markedGroup.Contains(i) || markedGroup.Contains(j))
                            {
                                break;
                            }
                            List<string> newGroup = new List<string>();
                            newGroup.AddRange(Groups[i]);
                            newGroup.AddRange(Groups[j]);
                            newGroups.Add(newGroup);
                            curMarkedGroup.Add(i);
                            curMarkedGroup.Add(j);

                            for (int k = 0; k < entropyForEachGroup.Count(); k++)
                            {
                                if (i < k)
                                {
                                    entropyForEachGroup[i][k] = 10000.0;
                                }
                                if (k < j)
                                {
                                    entropyForEachGroup[k][j] = 10000.0;
                                }
                                if (j < k)
                                {
                                    entropyForEachGroup[j][k] = 10000.0;
                                }
                                if (k < i)
                                {
                                    entropyForEachGroup[k][i] = 10000.0;
                                }
                            }
                            clustersCount--;
                            break;
                        }
                    }
                }

                for (int i = 0; i < Groups.Count; i++)
                {
                    for (int j = 0; j < newGroups.Count; j++)
                    {
                        if (curMarkedGroup.Contains(i) || markedGroup.Contains(i))
                        {
                            entropyForEachGroup[i].Add(10000.0);
                        }
                        else
                        {
                            entropyForEachGroup[i].Add(EntropyForGroup(Groups[i], newGroups[j]));
                        }
                    }
                }

                for (int i = 0; i < newGroups.Count; i++)
                {
                    List<double> curList = new List<double>();
                    for (int j = 0; j < Groups.Count; j++)
                    {
                        curList.Add(10000.0);
                    }
                    for (int j = 0; j < newGroups.Count; j++)
                    {
                        if (i < j)
                        {
                            curList.Add(EntropyForGroup(newGroups[i], newGroups[j]));
                        }
                        else
                        {
                            curList.Add(10000.0);
                        }
                    }
                    entropyForEachGroup.Add(curList);
                }
                Groups.AddRange(newGroups);
                foreach (int el in curMarkedGroup)
                {
                    if (markedGroup.Contains(el))
                    {
                        break;
                    }
                    markedGroup.Add(el);
                }

            }
            for (int i = 0; i < Groups.Count; i++)
            {
                if (!markedGroup.Contains(i))
                {
                    Clusters.Add(Groups[i]);

                    foreach (string el in Groups[i])
                    {
                        outputData4.Write(el + " ");
                        foreach (double doub in listOfVectorsCopy[listOfNameVectors.IndexOf(el)])
                        {
                            outputData4.Write(doub + " ");
                        }
                        outputData4.WriteLine();
                    }
                    outputData4.WriteLine();
                }
            }
            outputData4.Close();
        }

        private static void CreateGroups2()
        {
            //SearchInitialGr();
            List<string> clearGr = new List<string>();
            foreach (string query in listOfNameVectors)
            {
                List<string> qur = new List<string>();
                qur.Add(query);
                InitialGroups.Add(qur);
            }

            for (int i = 0; i < InitialGroups.Count; i++)
            {

                List<double> curList = new List<double>();
                for (int j = 0; j < InitialGroups.Count; j++)
                {
                    if (i < j)
                    {
                        curList.Add(EuclidDistance(InitialGroups[i], InitialGroups[j]));
                    }

                    else
                    {
                        curList.Add(10000.0);
                    }
                }
                distanceForEachQuery.Add(curList);
            }
            int clustersCount = InitialGroups.Count();
            List<int> markedGroup = new List<int>();
            while (clustersCount > 350)
            {
                //double min = 10000.0;

                List<List<string>> newGroups = new List<List<string>>();
                List<int> curMarkedGroup = new List<int>();

                bestFiveDistanceForEachQuery.Clear();
                List<double> sorted = new List<double>();
                for (int i = 0; i < distanceForEachQuery.Count(); i++)
                {

                    for (int j = 0; j < distanceForEachQuery.Count(); j++)
                    {
                        sorted.Add(distanceForEachQuery[i][j]);
                    }
                }
                sorted.Sort();
                List<double> curBest = new List<double>();
                int ih = 0;
                int kh = 0;
                while (ih < 1)
                {
                    if (!curBest.Contains(sorted[kh]))
                    {
                        curBest.Add(sorted[kh]);
                        ih++;
                    }
                    kh++;
                }
                bestFiveDistanceForEachQuery.AddRange(curBest);
                for (int i = 0; i < distanceForEachQuery.Count(); i++)
                {
                    for (int j = 0; j < distanceForEachQuery.Count(); j++)
                    {
                        if (i < j && bestFiveDistanceForEachQuery.Contains(distanceForEachQuery[i][j]))
                        {
                            List<string> newGroup = new List<string>();
                            newGroup.AddRange(InitialGroups[i]);
                            newGroup.AddRange(InitialGroups[j]);
                            newGroups.Add(newGroup);
                            if (curMarkedGroup.Contains(i) || curMarkedGroup.Contains(j) || markedGroup.Contains(i) || markedGroup.Contains(j))
                            {
                                break;
                            }
                            curMarkedGroup.Add(i);
                            curMarkedGroup.Add(j);
                            for (int k = 0; k < distanceForEachQuery.Count(); k++)
                            {
                                if (i < k)
                                {
                                    distanceForEachQuery[i][k] = 10000.0;
                                }
                                if (k < j)
                                {
                                    distanceForEachQuery[k][j] = 10000.0;
                                }
                                if (j < k)
                                {
                                    distanceForEachQuery[j][k] = 10000.0;
                                }
                                if (k < i)
                                {
                                    distanceForEachQuery[k][i] = 10000.0;
                                }
                            }
                            clustersCount--;
                            break;
                        }
                    }
                }

                for (int i = 0; i < InitialGroups.Count; i++)
                {
                    for (int j = 0; j < newGroups.Count; j++)
                    {
                        if (markedGroup.Contains(i) || curMarkedGroup.Contains(i))
                        {
                            distanceForEachQuery[i].Add(10000.0);
                        }
                        else
                        {
                            distanceForEachQuery[i].Add(EuclidDistance(InitialGroups[i], newGroups[j]));
                        }
                    }
                }

                for (int i = 0; i < newGroups.Count; i++)
                {
                    List<double> curList = new List<double>();
                    for (int j = 0; j < InitialGroups.Count; j++)
                    {
                        curList.Add(10000.0);
                    }
                    for (int j = 0; j < newGroups.Count; j++)
                    {
                        if (i < j)
                        {
                            curList.Add(EuclidDistance(newGroups[i], newGroups[j]));
                        }
                        else
                        {
                            curList.Add(10000.0);
                        }
                    }
                    distanceForEachQuery.Add(curList);
                }
                InitialGroups.AddRange(newGroups);
                foreach (int el in curMarkedGroup)
                {
                    if (markedGroup.Contains(el))
                    {
                        break;
                    }
                    markedGroup.Add(el);
                }
            }
            int count = 0;
            for (int i = 0; i < InitialGroups.Count; i++)
            {
                if (!markedGroup.Contains(i))
                {
                    Groups.Add(InitialGroups[i]);
                    foreach (string el in InitialGroups[i])
                    {
                        outputData3.Write(el + " ");
                        foreach (double doub in listOfVectors[listOfNameVectors.IndexOf(el)])
                        {
                            outputData3.Write(doub + " ");
                        }
                        outputData3.WriteLine();
                    }
                    outputData3.WriteLine();
                    count++;
                }
            }
            outputData3.Close();
        }
        static void clusterization()
        {
            foreach (List<string> group in Groups)
            {
                if (group.Count() > 1)
                {
                    InitialGroups.Add(group);
                    markedVectors.AddRange(group);
                }
            }
            foreach (string vector in listOfNameVectors)
            {
                if (!markedVectors.Contains(vector))
                {
                    List<double> quality = new List<double>();
                    for (int k = 0; k < InitialGroups.Count(); k++)
                    {
                        InitialGroups[k].Add(vector);
                        double entropy = EntropyOfClustering(InitialGroups);
                        quality.Add(entropy);
                        InitialGroups[k].Remove(vector);
                    }
                    InitialGroups[quality.IndexOf(quality.Min())].Add(vector);
                    markedVectors.Add(vector);
                }

            }
            for (int i = 0; i < InitialGroups.Count; i++)
            {

                Clusters.Add(InitialGroups[i]);

                foreach (string el in InitialGroups[i])
                {
                    outputData4.Write(el + " ");
                    foreach (double doub in listOfVectorsCopy[listOfNameVectors.IndexOf(el)])
                    {
                        outputData4.Write(doub + " ");
                    }
                    outputData4.WriteLine();
                }
                outputData4.WriteLine();

            }
            outputData4.Close();
        }
        private static double EntropyOfClustering(List<List<string>> groups)
        {
            double result = 0.0;
            int allweight = 0;
            foreach (List<string> group in groups)
            {
                allweight += group.Count();
            }
            int numerOfComponents = 7;
            foreach (List<string> group in groups)
            {
                List<string> marked = new List<string>();
                int numberOfElements = group.Count();
                double[] entropyArray = new double[numerOfComponents];
                List<int> frequencyArray = new List<int>();
                for (int i = 0; i < numerOfComponents; i++)
                {
                    marked.Clear();
                    frequencyArray.Clear();
                    for (int j = 0; j < numberOfElements; j++)
                    {
                        if (!marked.Contains(group[j]))
                        {
                            int curFriq = 1;
                            marked.Add(group[j]);

                            for (int k = 0; k < numberOfElements; k++)
                            {
                                if (!marked.Contains(group[k]))
                                {
                                    if (listOfVectors[listOfNameVectors.IndexOf(group[j])][i] == listOfVectors[listOfNameVectors.IndexOf(group[k])][i])
                                    {
                                        curFriq++;
                                        marked.Add(group[k]);
                                    }
                                }
                            }
                            frequencyArray.Add(curFriq);
                        }
                    }
                    foreach (int frequency in frequencyArray)
                    {
                        entropyArray[i] += -(((float)frequency / numberOfElements) * Math.Log(((float)frequency / numberOfElements), 2));
                    }
                }
                double weight = (float)group.Count() / allweight;
                result += entropyArray.Sum() * weight;
            }
            return result;
        }
        static void Main(string[] args)
        {
            CreateVectors();
            foreach (List<double> el in listOfVectors)
            {
                listOfVectorsCopy.Add(el);
            }
            listOfVectors = new List<List<double>>();
            for (int j = 0; j < listOfVectorsCopy.Count(); j++)
            {
                List<double> curList = new List<double>();
                for (int i = 0; i < 7; i++)
                {
                    curList.Add(Math.Round(listOfVectorsCopy[j][i], 2));

                }
                listOfVectors.Add(curList);
            }
            Create();
            clusterization();
            foreach (List<double> el in markedCenters)
            {
                foreach (double doub in el)
                {
                    Console.Write(doub + " ");
                }
                Console.WriteLine();
            }

            CreateCenters();
            PrintCenters();

            outputData.Close();
        }
    }
}
